#!/bin/bash
#f13toattr63.bash
#
#Taylor Asher 2012-05-15
#Updated 2014-03-25 to allow for processing of all attributes at once

function attrto63
{
#Turn single-line of default values into an array of default values
#This needs to be done if $nvals is greater than 1
ctr2=1
for i in $defvals ; do
   arr[$ctr2]=$i
   ctr2=$((ctr2+1))
done
defvals=("${arr[@]}")

#If the nodal attribute name isn't the one you want, read through it to get 
#to the next one.  If it is what you want, then read it and dump it to a 
#temporary file $tmpfil, keeping the $ndifnod variable for later, then 
#break out of the for loop since there's no need to keep reading the file
if [[ "$curnanam" != $nanam ]] ; then
   for ctr2 in $(seq 1 $curndifnod) ; do
      read <&$fidin dum
   done
else
   echo "Processing attribute $curnanam"
   ndifnod=$curndifnod
   for ctr2 in $(seq 1 $ndifnod) ; do
      read <&$fidin dum
      echo $dum >>$tmpfil
   done
fi


#Open up output file(s), $nvals files total, and put fort.63-style header info 
#into each one.  Then, cut columns out of $tmpfil and put them into the target 
#sparse output file.  Lastly, run the inflate.x FORTRAN routine to inflate the
#sparse versions of the output files.  Sparse files are then deleted.  
for ctr2 in $(seq 1 $nvals) ; do
   sparsefilout[$ctr2]="sparse$filoutpat$ctr2.63"                              #name of sparse output file
   filout[$ctr2]="$filoutpat$ctr2.63"                                          #name of full output file
   #Write 3 fort.63-style header lines to files
   echo $filinfo $nanam > "${sparsefilout[$ctr2]}"
   echo "1 $nn 1 1 1" >> "${sparsefilout[$ctr2]}"
   echo "1 1 $ndifnod ${defvals[$((ctr2-1))]}" >> "${sparsefilout[$ctr2]}"
   #Grab 1st and $ctr2-th columns from the temporary nodal attribute file and 
   #spit them out to the sparse output file
   cut -d ' ' -f "1 $((ctr2+1))" $tmpfil >>"${sparsefilout[$ctr2]}"
   ./inflate.x ${sparsefilout[$ctr2]} ${filout[$ctr2]}                         #inflate the sparse output file to the full one
   rm ${sparsefilout[$ctr2]}
   echo ${sparsefilout[$ctr2]}
done
rm $tmpfil
}




#input file name and file identifier
#samples:  
#filin=fort.13
#nanam="surface_directional_effective_roughness_length"
#filoutpat=SDERL
filin=$1                               #input file name
filoutpat=$2                           #output file base name (ignored if $nanam==--all)
nanam=$3                               #nodal attribute name (or --all to process all attributes)
fidin=100                              #file ID assigned to input file
tmpfil=tmpf13toattr63.tmp              #name of temporary file to be used during run

echo "Begin $0"

#Open file
echo "Opening input file $filin"
eval "exec $fidin<$filin"

#Read in file info, number of nodes, and number of nodal attributes
#The "dum" part of the read is to ensure any additional info on the line is 
#written to a dummy variable
read <&$fidin filinfo
read <&$fidin nn dum
read <&$fidin na dum

#If all attributes are to be processed, do appropriate preps
if [ "$nanam" == '--all' ]; then
   doall='y'
   nattrdo=$na
else
   doall='n'
   nattrdo=1
fi

#Loop to read in header areas of fort.13 file and identify number of values 
#per line in target attribute and default value
echo "Identifying target nodal attribute $nanam"
for ctr1 in $(seq 1 $na) ; do
   read <&$fidin curnanam                        #current nodal attribute name
   read <&$fidin curunits                        #current units
   read <&$fidin curnvals                        #current number of attribute values per node
   read <&$fidin curdefvals                      #current default value(s)

   #Assign "current" values to "permanent" variables if the target nodal
   #attribute is found or if doing all attributes
   if [ $doall == 'y' ]; then
      listnanam[$ctr1]=$curnanam
      listunits[$ctr1]=$curunits
      listnvals[$ctr1]=$curnvals
      listdefvals[$ctr1]=$curdefvals
      listfiloutpat[$ctr1]=$curnanam
   elif [[ "$curnanam" == $nanam ]] ; then
      units=$curunits
      nvals=$curnvals
      defvals=$curdefvals
   fi
done


#Get rid of $tmpfil if it exists, as we don't want anything to already be inside
if [ -f $tmpfil ] ; then
   echo "Removing pre-existing temp file $tmpfil"
   rm $tmpfil
fi

#Read through file until you find parameter of interest, then export nodal 
#attribute contents to a new temporary file, $tmpfil, for further processing
echo "Reading through input file $filin"
for ctr1 in $(seq 1 $na) ; do
   read <&$fidin curnanam                           #current nodal attribute name
   read <&$fidin curndifnod                         #current number of attribute that aren't the default value
   if [ $doall == 'y' ]; then
      attrcnt=0
      nanam=$curnanam
      for i in ${listnanam[*]}; do
         attrcnt=$((attrcnt+1))
         if [[ $i == $curnanam ]]; then
            units=${listunits[$attrcnt]}
            nvals=${listnvals[$attrcnt]}
            defvals=${listdefvals[$attrcnt]}
            filoutpat=${listfiloutpat[$attrcnt]}
            break
         fi
      done
   fi
   attrto63
done

echo "Done $0"
